; Jet Hopper V1.0
; By Robin Kay
;

; You may not distribute a modified version of this program. However, you
; may take code from or base your program on this code providing that the
; resulting program is at least 50% of your own work.

.nolist
#include "tse.inc"
.list
.org    usermem-3
.db     $bb,$6d
 ret

.db "TSE",1              ; header
prgtitle:
.db " Jet Hopper", 0     ; title
.dw 69                   ; stack required

start:

 ; Dispaly main menu
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl 
 ld hl, prgtitle + 1
 call putStr
 ld hl, author
 call putStr
 bcall(_newline)
 ld hl, txt1
 call putStr
 ld hl, txt2
 call putStr
 bcall(_newline)
 bcall(_newline)
 ld hl, hstxt
 bcall(_puts)
 ld hl, (highscore)
 bcall(_disphl)

 ; Poll for keypad on menu
loop:
 ei
 halt ; Save power
 bcall(_getcsc)

 cp sk1
 jr nz, cont0
 ld h, 15
 jr maingame
cont0:

 cp sk2
 jr nz, cont1
 ld h, 10
 jr maingame
cont1:

 cp sk3
 jr nz, cont2
 ld h, 5
 jr maingame
cont2:

 ; Process other keys
 cp skclear
 jr z, clrhs

 ; Yield
 cp skStat
 jr nz, dontyield
 call _tseforceyield
 jr start
dontyield:
 jr loop

; Clear highscore
clrhs:
 ld hl, 0
 ld (highscore), hl
 jr start 

; Main game routine
maingame:

 ; Save delay
 ld l, 0
 ld (delay), hl

 ; Reset Score
 ld h, 0
 ld (score), hl

 ; Init vars
 ld a, 32
 ld (hoppery), a
 xor a
 ld (pjump), a
 ld a, 8
 ld (offset), a

 ; Init platforms
 ld hl, platforms
 ld b, 11
 ld a, 63-5
setuploop:
 ld (hl), a
 inc hl
 djnz setuploop

loopgame:

 ; Clear the screen
 bcall(_GrBufClr)

 ; Display hopper
 ld ix, hopper
 ld b, 5
 ld a, (hoppery)
 ld l, a
 ld a, 8
 call ionPutSprite

 ; Display platforms
 ld hl, platforms
 ld b, 11
drawloop:
 push hl
 push bc
 ld l, (hl)
 dec b
 sla b
 sla b
 sla b
 ld a, (offset)
 add a, b
 ld b, 5
 ld ix, platform
 call ionPutSprite
 pop bc
 pop hl
 inc hl
 djnz drawloop

 ; Draw onto screen
 call ionFastCopy

 ; Advance offset on
 ld a, (offset)
 dec a
 cp -1
 jr nz, addoffset

 ; Advance platforms on
 ld a, 8
 ld bc, 10
 ld hl, platforms+9
 ld de, platforms+10
 lddr
 push af
 ld b, 2
 call ionRandom
 cp 0
 jr nz, dont
 ld b, 9
 call ionRandom
 sub 4
 sla a
 ld b, a
 ld a, (platforms)
 add a, b
 cp 16
 jr c, dont
 cp 63-5
 jr nc, dont
 ld (platforms), a
dont:
 pop af
addoffset:
 ld (offset), a

 ; Calculate lowest
 ld a, (platforms+9)
 add a, -5
 ld (lowest), a
 ld b, a
 ld a, (hoppery)
 dec a
 cp b
 jp nc, done

 ; Poll keypad
 ld a,KReset
 out (1),a
 ld a,group7
 out (1),a
 in a,(1)
 cp k2nd
 jr nz, donekey
 ld a, (pjump)
 cp 0
 jr z, donekey
 dec a
 ld (pjump), a
 ld a, (hoppery)
 add a, -3
 cp 8
 jr c, donekey
 ld (hoppery), a
donekey:

 ; Gravity
 ld a, (lowest)
 ld b, a
 ld a, (hoppery)
 cp b
 jr z, nofall
 inc a
 ld (hoppery), a
 jr fall
nofall:
 ld a, 15
 ld (pjump), a
fall:

 ; Delay
 ld bc, (delay)
 ld hl, 8000h
 ld d, h
 ld e, l
 ldir ; Nop!

 ; Increase score
 ld hl, (score)
 inc hl
 ld (score), hl

 ; Yield
 ld a,kreset
 out (1),a
 ld a,group4
 out (1),a
 in a,(1)
 cp KStat
 jp nz, loopgame
 call _tseforceyield
 jp loopgame
                 
; Display string and then advance a line onwards
putStr:
 bcall(_puts)
 bcall(_newline)
 ret

; Hiscore screen
done:
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, scrtxt
 bcall(_puts)
 ld hl, (score)
 bcall(_disphl)
 bcall(_newline)
 ld hl, (score)
 ld de, (highscore)
 inc de
 bcall(_cphlde)
 jr c, nohighscore

 ld (highscore), hl
 ld hl, newhs
 bcall(_puts)

nohighscore:
 ei
 halt ; Save power
 bcall(_getcsc)
 cp skMode
 jp z, start
 cp skStat
 jr nz, nohighscore
 call z, _tseforceyield
 jr done

; Hopper Sprite
hopper:
.db %01111110
.db %11111111
.db %01111110
.db %00100010
.db %01010101

platform:
.db %11111111
.db %11111111
.db %11101110
.db %01000100
.db %01000100

highscore:
.dw 0

author:
.db "By Robin Kay", 0
txt1:
.db "1-3: Start Game",0
txt2:
.db "Clr: Reset HS",0
hstxt:
.db "Hi-"
scrtxt
.db "Score: ",0
newhs:
.db "New HighScore!!",0

extdata:
delay        .equ extdata
score        .equ delay+2
hoppery      .equ score+2
pjump        .equ hoppery+1
platforms    .equ pjump+1
offset       .equ platforms+11
lowest       .equ offset+1

.end
